"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@babel/core");
const plugin_syntax_jsx_1 = require("@babel/plugin-syntax-jsx");
const constants_1 = require("./constants");
exports.default = (_, opts = {}) => {
    opts.static = typeof opts.static === 'boolean' ? opts.static : true;
    opts.strict = typeof opts.strict === 'boolean' ? opts.strict : true;
    opts.importSource = opts.importSource || constants_1.IMPORT_SOURCE;
    opts.importName = opts.importName || constants_1.IMPORT_NAME;
    const callId = core_1.types.identifier(constants_1.IMPORT_NAMESPACE);
    const importSpec = opts.importName === constants_1.IMPORT_NAME
        ? core_1.types.importDefaultSpecifier(callId)
        : core_1.types.importSpecifier(callId, core_1.types.identifier(opts.importName));
    const importDecl = core_1.types.importDeclaration([importSpec], core_1.types.stringLiteral(opts.importSource));
    function isIgnoredGlobal(nodes) {
        for (const node of nodes) {
            if (core_1.types.isImportDeclaration(node)) {
                if (isIgnored(node, constants_1.CLSX_IGNORE_GLOBAL_TOKEN))
                    return true;
            }
            else {
                return isIgnored(node, constants_1.CLSX_IGNORE_GLOBAL_TOKEN);
            }
        }
        return false;
    }
    function isIgnored(node, token = constants_1.CLSX_IGNORE_TOKEN) {
        return node.leadingComments
            ? node.leadingComments.some((comment) => {
                const ignored = comment.value.trim() === token;
                if (ignored)
                    comment.ignore = ignored;
                return ignored;
            })
            : false;
    }
    function isNeedTransform(jsxExpr) {
        if (opts.static) {
            return (core_1.types.isArrayExpression(jsxExpr.expression) ||
                core_1.types.isObjectExpression(jsxExpr.expression));
        }
        else {
            return (!core_1.types.isCallExpression(jsxExpr.expression) &&
                !core_1.types.isStringLiteral(jsxExpr.expression));
        }
    }
    const classNameRE = opts.strict ? constants_1.CLASS_NAME_STRICT_RE : constants_1.CLASS_NAME_RE;
    function isDynamicClassName(node) {
        return (classNameRE.test(node.name.name) &&
            core_1.types.isJSXExpressionContainer(node.value));
    }
    function replaceNode(path) {
        const args = (core_1.types.isArrayExpression(path.node) ? path.node.elements : [path.node]);
        const callExpr = core_1.types.callExpression(callId, args);
        path.replaceWith(callExpr);
    }
    return {
        name: 'clsx',
        inherits: plugin_syntax_jsx_1.default,
        visitor: {
            Program: {
                enter(path, state) {
                    state.clsxIgnoreGlobal = isIgnoredGlobal(path.node.body);
                },
                exit(path, state) {
                    if (state.clsxImport) {
                        path.node.body.unshift(importDecl);
                    }
                },
            },
            JSXAttribute(path, state) {
                if (isDynamicClassName(path.node) &&
                    !isIgnored(path.node) &&
                    !state.clsxIgnoreGlobal &&
                    isNeedTransform(path.node.value)) {
                    state.clsxImport = true;
                    replaceNode(path.get('value').get('expression'));
                }
            },
        },
    };
};
